@@ -125,6 +125,18 @@ QString ConfigValueProvider::cursorThemeName() const
     return configGroup.readEntry(QStringLiteral("cursorTheme"), QStringLiteral("breeze_cursors"));
 }
 
+QString ConfigValueProvider::soundThemeName() const
+{
+    KConfigGroup configGroup = kdeglobalsConfig->group(QStringLiteral("Sounds"));
+    return configGroup.readEntry(QStringLiteral("Theme"), QStringLiteral("ocean"));
+}
+
+bool ConfigValueProvider::eventSoundsEnabled() const
+{
+    KConfigGroup configGroup = kdeglobalsConfig->group(QStringLiteral("Sounds"));
+    return configGroup.readEntry(QStringLiteral("Enable"), true);
+}
+
 int ConfigValueProvider::cursorSize() const
 {
     KConfigGroup configGroup = inputConfig->group(QStringLiteral("Mouse"));
@@ -162,7 +174,7 @@ int ConfigValueProvider::toolbarStyle() const
 bool ConfigValueProvider::scrollbarBehavior() const
 {
     KConfigGroup configGroup = kdeglobalsConfig->group(QStringLiteral("KDE"));
-    bool kdeConfigValue = configGroup.readEntry(QStringLiteral("ScrollbarLeftClickNavigatesByPage"), true);
+    bool kdeConfigValue = configGroup.readEntry(QStringLiteral("ScrollbarLeftClickNavigatesByPage"), false);
     return !kdeConfigValue; // GTK setting is inverted
 }
 
@@ -178,14 +190,14 @@ bool ConfigValueProvider::preferDarkTheme() const
 
 QStringList ConfigValueProvider::windowDecorationsButtonsImages() const
 {
-    static const QVector<QString> buttonTypes{
+    static const QList<QString> buttonTypes{
         QStringLiteral("close"),
         QStringLiteral("maximize"),
         QStringLiteral("maximized"),
         QStringLiteral("minimize"),
     };
 
-    static const QVector<QString> buttonStates{
+    static const QList<QString> buttonStates{
         // Focused titlebars
         QStringLiteral("normal"),
         QStringLiteral("active"), // aka pressed
@@ -242,6 +254,12 @@ bool ConfigValueProvider::enableAnimations() const
     return !qFuzzyIsNull(animationSpeedModifier);
 }
 
+int ConfigValueProvider::doubleClickInterval() const
+{
+    KConfigGroup generalCfg = kdeglobalsConfig->group("KDE");
+    return generalCfg.readEntry("DoubleClickInterval", 400);
+}
+
 QMap<QString, QColor> ConfigValueProvider::colors() const
 {
     using KCS = KColorScheme;
