@@ -64,12 +64,23 @@ GtkConfig::~GtkConfig()
     dbus.unregisterObject(QStringLiteral("/GtkConfig"));
 }
 
+void GtkConfig::setGtk2Theme(const QString &themeName, const bool preferDarkTheme) const
+{
+    // GTK2 does not support using dark variant automatically, so we have to get dark preference and switch based on that
+    QString possiblydarkthemeName = themeName;
+    if (themeName == QLatin1String("Breeze") && preferDarkTheme) {
+        possiblydarkthemeName = QStringLiteral("Breeze-Dark");
+    }
+
+    Gtk2ConfigEditor::setValue(QStringLiteral("gtk-theme-name"), possiblydarkthemeName);
+    XSettingsEditor::setValue(QStringLiteral("Net/ThemeName"), possiblydarkthemeName);
+}
+
 void GtkConfig::setGtkTheme(const QString &themeName) const
 {
-    Gtk2ConfigEditor::setValue(QStringLiteral("gtk-theme-name"), themeName);
+    setGtk2Theme(themeName, configValueProvider->preferDarkTheme());
     GSettingsEditor::setValue("gtk-theme", themeName);
     SettingsIniEditor::setValue(QStringLiteral("gtk-theme-name"), themeName);
-    XSettingsEditor::setValue(QStringLiteral("Net/ThemeName"), themeName);
 
     // Window decorations are part of the theme, in case of Breeze we inject custom ones from KWin
     setWindowDecorationsAppearance();
@@ -103,6 +114,24 @@ void GtkConfig::setIconTheme() const
     XSettingsEditor::setValue(QStringLiteral("Net/IconThemeName"), iconThemeName);
 }
 
+void GtkConfig::setSoundTheme() const
+{
+    const QString soundThemeName = configValueProvider->soundThemeName();
+    Gtk2ConfigEditor::setValue(QStringLiteral("gtk-sound-theme-name"), soundThemeName);
+    GSettingsEditor::setValue("theme-name", soundThemeName, "org.gnome.desktop.sound");
+    SettingsIniEditor::setValue(QStringLiteral("gtk-sound-theme-name"), soundThemeName);
+    XSettingsEditor::setValue(QStringLiteral("Net/SoundThemeName"), soundThemeName);
+}
+
+void GtkConfig::setEventSoundsEnabled() const
+{
+    const bool soundsEnabled = configValueProvider->eventSoundsEnabled();
+    Gtk2ConfigEditor::setValue(QStringLiteral("gtk-enable-event-sounds"), soundsEnabled);
+    GSettingsEditor::setValue("event-sounds", soundsEnabled, "org.gnome.desktop.sound");
+    SettingsIniEditor::setValue(QStringLiteral("gtk-enable-event-sounds"), soundsEnabled);
+    XSettingsEditor::setValue(QStringLiteral("Net/EnableEventSounds"), soundsEnabled);
+}
+
 void GtkConfig::setCursorTheme() const
 {
     const QString cursorThemeName = configValueProvider->cursorThemeName();
@@ -114,11 +143,16 @@ void GtkConfig::setCursorTheme() const
 
 void GtkConfig::setCursorSize() const
 {
+    qreal xwaylandScale = 1.0;
+    if (KWindowSystem::isPlatformWayland()) {
+        xwaylandScale = configValueProvider->x11GlobalScaleFactor();
+    }
+
     const int cursorSize = configValueProvider->cursorSize();
     Gtk2ConfigEditor::setValue(QStringLiteral("gtk-cursor-theme-size"), cursorSize);
     GSettingsEditor::setValue("cursor-size", cursorSize);
     SettingsIniEditor::setValue(QStringLiteral("gtk-cursor-theme-size"), cursorSize);
-    XSettingsEditor::setValue(QStringLiteral("Gtk/CursorThemeSize"), cursorSize);
+    XSettingsEditor::setValue(QStringLiteral("Gtk/CursorThemeSize"), int(cursorSize * xwaylandScale));
 }
 
 void GtkConfig::setIconsOnButtons() const
@@ -157,6 +191,15 @@ void GtkConfig::setScrollbarBehavior() const
     XSettingsEditor::setValue(QStringLiteral("Gtk/PrimaryButtonWarpsSlider"), scrollbarBehavior);
 }
 
+void GtkConfig::setDoubleClickInterval() const
+{
+    const int doubleClickInterval = configValueProvider->doubleClickInterval();
+    Gtk2ConfigEditor::setValue(QStringLiteral("gtk-double-click-time"), doubleClickInterval);
+    GSettingsEditor::setValue("double-click", doubleClickInterval, "org.gnome.desktop.peripherals.mouse");
+    SettingsIniEditor::setValue(QStringLiteral("gtk-double-click-time"), doubleClickInterval);
+    XSettingsEditor::setValue(QStringLiteral("Net/DoubleClickTime"), doubleClickInterval);
+}
+
 void GtkConfig::setDarkThemePreference() const
 {
     const bool preferDarkTheme = configValueProvider->preferDarkTheme();
@@ -165,6 +208,7 @@ void GtkConfig::setDarkThemePreference() const
     GSettingsEditor::setValueAsEnum("color-scheme",
                                     preferDarkTheme ? 1 /*G_DESKTOP_COLOR_SCHEME_PREFER_DARK*/ : 0 /*G_DESKTOP_COLOR_SCHEME_DEFAULT*/,
                                     "org.gnome.desktop.interface");
+    setGtk2Theme(gtkTheme(), preferDarkTheme);
 }
 
 void GtkConfig::setWindowDecorationsAppearance() const
@@ -197,14 +241,9 @@ void GtkConfig::setEnableAnimations() const
     }
 }
 
-static double roundPreferFloor(double d)
-{
-    return (d - floor(d)) > 0.5 ? ceil(d) : floor(d);
-}
-
 void GtkConfig::setGlobalScale() const
 {
-    const unsigned scaleFactor = roundPreferFloor(configValueProvider->x11GlobalScaleFactor());
+    const unsigned scaleFactor = configValueProvider->x11GlobalScaleFactor();
     XSettingsEditor::setValue(QStringLiteral("Gdk/WindowScalingFactor"), scaleFactor);
     GSettingsEditor::setValue("scaling-factor", scaleFactor); // For IntelliJ IDEA
 }
@@ -212,7 +251,7 @@ void GtkConfig::setGlobalScale() const
 void GtkConfig::setTextScale() const
 {
     const double x11Scale = configValueProvider->x11GlobalScaleFactor();
-    const int x11ScaleIntegerPart = int(roundPreferFloor(x11Scale));
+    const int x11ScaleIntegerPart = int(x11Scale);
 
     const int forceFontDpi = configValueProvider->fontDpi();
 
@@ -255,6 +294,7 @@ void GtkConfig::applyAllSettings() const
 {
     setFont();
     setIconTheme();
+    setSoundTheme();
     setCursorTheme();
     setCursorSize();
     setIconsOnButtons();
@@ -282,13 +322,25 @@ void GtkConfig::onKdeglobalsSettingsChange(const KConfigGroup &group, const QByt
         if (names.contains(QByteArrayLiteral("ShowIconsOnPushButtons"))) {
             setIconsOnButtons();
         }
-        if (names.contains(QByteArrayLiteral("ScrollbarLeftClickNavigatesByPage"))) {
+        // ScrollbarLeftClickNavigatesByPage is now the default setting, so when it's
+        // true, it won't be present, so we need to check for its absence
+        if (!names.contains(QByteArrayLiteral("ScrollbarLeftClickNavigatesByPage"))) {
             setScrollbarBehavior();
         }
+        if (names.contains(QByteArrayLiteral("DoubleClickInterval"))) {
+            setDoubleClickInterval();
+        }
     } else if (group.name() == QStringLiteral("Icons")) {
         if (names.contains(QByteArrayLiteral("Theme"))) {
             setIconTheme();
         }
+    } else if (group.name() == QLatin1String("Sounds")) {
+        if (names.contains(QByteArrayLiteral("Theme"))) {
+            setSoundTheme();
+        }
+        if (names.contains(QByteArrayLiteral("Enable"))) {
+            setEventSoundsEnabled();
+        }
     } else if (group.name() == QStringLiteral("General")) {
         if (names.contains(QByteArrayLiteral("font"))) {
             setFont();
@@ -324,6 +376,7 @@ void GtkConfig::onKWinSettingsChange(const KConfigGroup &group, const QByteArray
         if (names.contains(QByteArrayLiteral("Scale"))) {
             setGlobalScale();
             setTextScale();
+            setCursorSize();
         }
     }
 }
@@ -358,3 +411,5 @@ void GtkConfig::onBreezeSettingsChange(const KConfigGroup &group, const QByteArr
 }
 
 #include "gtkconfig.moc"
+
+#include "moc_gtkconfig.cpp"
